<?php
/**
 * Project:	ePrint :: Extended Print API
 * File:	ePrint.php
 *
 * PHP version 5
 *
 * Copyright (c) 1997-2009 JoungKyun.Kim
 *
 * LICENSE: BSD license
 *
 * @category	Core
 * @package		ePrint
 * @author		JoungKyun.Kim <http://oops.org>
 * @copyright	1997-2009 OOPS.ORG
 * @license		BSD License
 * @version		CVS: $Id: ePrint.php,v 1.11 2010/05/25 10:13:11 oops Exp $
 * @link		http://pear.oops.org/package/ePrint
 * @since		File available since relase 1.0.0
 */

/**
 * Base classes for Extended Print API
 * @package		ePrint
 */
class ePrint {
	// {{{ properties
	const EPRINT_NORMAL  = 0;
	const EPRINT_WARNING = 1;
	const EPRINT_ERROR   = 2;

	/**
	 * Set Standard Input file description
	 *
	 * @var		resource
	 * @access	public
	 */
	static public $stdin;
	/**
	 * Set Standard Out file description
	 *
	 * @var		resource
	 * @access	public
	 */
	static public $stdout;
	/**
	 * Set Standard Error file description
	 *
	 * @var		resource
	 * @access	public
	 */
	static public $stderr;

	/**
	 * Set default debug level for ePrint::dprintf api
	 *
	 * @var		int
	 * @access	public
	 */
	static public $debugLevel = 0;
	static public $debugIP;
	// }}}

	// {{{ constructor
	/**
	 * @access public
	 * @return void
	 */
	function __construct () {
		self::regist ();
	}
	// }}}

	// {{{ function regist ($force = false)
	/**
	 * Regist stdin/stdout/stderr file descriptor
	 * This api runs only cli mode
	 * @access	public
	 * @return	void
	 * @param	boolean	(optional) Defaults to false. On no cli mode, set ture,
	 * 					force runs.
	 */
	function regist ($force = false) {
		if ( php_sapi_name () == 'cli' )
			$force = true;

		if ( $force === false )
			return;

		self::$stdin = fopen ('php://stdin', 'rb');
		self::$stdout = fopen ('php://stdout', 'wb');
		self::$stderr = fopen ('php://stderr', 'wb');
	}
	// }}}

	// {{{ function whiteSpace ($no, $ret = false)
	/**
	 * Print white space about given number
	 * @access	public
	 * @return	void	void. If $ret set true, return strings.
	 * @param	integer	number of space charactor
	 * @param	boolean	(optional) Set true, return space charactors,
	 * 					and, don't print out. Defaults to false.
	 */
	function whiteSpace ($no, $ret = false) {
		if ( ! is_numeric ($no) )
			return '';

		if ( $ret === false ) {
			for ( $i=0; $i<$no; $i++ )
				echo ' ';
			return;
		}

		for ( $i=0; $i<$no; $i++ )
			$_r.= ' ';

		return $_r;
	}
	// }}}

	// {{{ function backSpace ($no, $handle = null)
	/**
	 * print backspace
	 * @access	public
	 * @return	void
	 * @param	integer	number of space charactor
	 * @param	resource (optional) A file system pointer resource that is typically created using fopen().
	 */
	function backSpace ($no, $handle = null) {
		if ( ! is_numeric ($no) )
			return '';

		for ( $i=0; $i<$no; $i++ ) {
			if ( $handle === null )
				printf ('%c %c', '008', '008');
			else
				fprintf ($handle, '%c %c', '008', '008');
		}
		/*
		for ( $i=0; $i<$no; $i++ ) {
			echo "12" ^ "9";
			echo ' ';
			echo "12" ^ "9";
		}
		 */
	}
	// }}}

	// {{{ function ePrintf ($format, $msg = '', $type = 0, $des = '', $extra_header = '')
	/**
	 * Output a formatted string to stderr
	 * @access	public
	 * @return	int		length of output messages.
	 * @param	string	same format of printf
	 * @param	mixed	(optional) format value.<br>
	 * 					If only one format argument, $msg is strings or array.<br>
	 * 					For multiple format arguments, $msg is array.
	 * @param	integer	(optional) this is 3th argument of error_log
	 * @param	string	(optional) this is 4th argument of error_log
	 * @param	string	(optional) this is 5th argument of error_log
	 */
	function ePrintf ($format, $msg = '', $type = 0, $des = '', $extra_header = '') {
		if ( $msg ) {
			$functionName = is_array ($msg) ? 'vsprintf' : 'sprintf';
			$format = $functionName ($format, $msg);
		}

		return error_log ($format, $type, $des, $extra_headers);
	}
	// }}}

	// {{{ function dPrintf ()
	/**
	 * Output a formatted string for debug. This is debug printf.
	 * @access	public
	 * @return	int		length of print string
	 * @param	integer	$level	if ePrint::$debug_level is bigger than $level or same, print out.
	 * @param	string	$format	same format of printf
	 * @param	string	$args	(optional) format value.
	 * @param	string	$...	(optional) format value.
	 */
	function dPrintf () {
		$args= func_get_args ();
		if ( count ($args) < 2 ) {
			self::warning ('Wrong parameter count for ePrint::dPrintf()');
			return;
		}

		$level = array_shift ($args);
		if ( self::$debugLevel < $level )
			return;

		return call_user_func_array (printf, $args);
	}
	// }}}

	// {{{ function lPrintf ()
	/**
	 * Save a formatted string to file
	 * A newline is not automatically added to the end of the message string. 
	 * @access	public
	 * @return	int		length of print string
	 * @param	string	$path	target file
	 * @param	string	$format	same format of printf
	 * @param	string	$args	(optional) format value.
	 * @param	string	$...	(optional) format value.
	 */
	function lPrintf () {
		$args= func_get_args ();
		if ( count ($args) < 2 ) {
			self::warning ('Wrong parameter count for ePrint::lPrintf()');
			return;
		}

		$f = array_shift ($args);
		$format = array_shift ($args);
		return self::ePrintf ($format, $args, 3, $f);
	}
	// }}}

	// {{{ function fprintf () {
	/*
	function fprintf () {
		$args= func_get_args ();

		if ( count ($args) < 2 ) {
			self::warning ('Wrong parameter count for ePrint::fprintf()');
			return;
		}

		if ( function_exists ('fprintf') )
			return call_user_func_array (fprintf, $args);

		$handle =  $array_shift ($args);
		if ( ! is_resource ($handle) ) {
			self::warning ('fprintf() supplied argument is not a valid stream resource');
			return;
		}
		$format = array_shift ($args);

		return fwrite ($handle, vsprintf ($format, $args));
	}
	 */
	// }}}

	// {{{ function printi ($msg, $width = 80, $indent = 0, $ul = '', $to_stderr = false)
	/**
	 * print with indent.
	 * @access	public
	 * @return	int|false	Length of print string. If on error, return false
	 * @param   string|array	output string
	 * @param   integer	(optional) location of line brek. default 80
	 * @param   integer	(optional) indent of each line
	 * @param   string	(optional) list itme
	 * @param	boolean	(optional) set true, print stderr
	 */
	function printi ($msg, $width = 80, $indent = 0, $ul = '', $to_stderr = false) {
		if ( $width === 0 )
			$width = 10000; /* means unlimits */

		if ( $indent > 0 ) {
			$width -= $indent;
			if ( $width <= 0 ) {
				self::warning ('Plez make string lenght more than current length for ePrint::wordwrap()');
				return false;
			}
		}

		$l = strlen ($ul);
		if ( $l > 0 ) {
			$width -= $l + 1;
			$ul .= ' ';
		}

		if ( is_array ($msg) ) {
			if ( !count ($msg) )
				return;

			foreach ($msg as $v)
				$_msg .= rtrim ($v) . "\n";
		} else {
			if ( ! trim ($msg) )
				return;

			$_msg = preg_replace ('/[\r]/', '', $msg);
		}

		unset ($msg);
		$msg = rtrim (self::wordwrap ($_msg, $width));
		if ( preg_match ('/-newline$/', $msg) ) {
			$msg = preg_replace ('/-newline$/', '', $msg);
			$new_line = 0;
		} else {
			$new_line = 1;
		}

		unset ($_msg);
		$_msg = split ("[\n]", $msg);
		$_line = count ($_msg);

		for ( $i=0; $i<$_line; $i++ ) {
			if ( $i == 1 ) {
				$_ul = strlen ($ul);
				$ul = str_repeat (' ', $_ul);
			}
			if ( $to_stderr ) {
				self::ePrintf ("%s%s%s", array (str_repeat (' ', $indent), $ul, $_msg[$i]));
			} else {
				printf ("%s%s%s%s",
						str_repeat (' ', $indent),
						$ul,
						str_replace ('&nbsp;', ' ', $_msg[$i]),
						$new_line ? "\n" : '');
			}
		}

		return true;
	}
	// }}}

	// {{{ function echoi {$msg, $indent = 0)
	/**
	 * echo with indent.
	 * @access	public
	 * @return	void
	 * @param   string|array	output string
	 * @param   integer	(optional) indent of each line. Defaults to 0
	 */
	function echoi ($msg, $indent = 0) {
		if ( ! is_array ($msg) ) {
			$msg = preg_replace ('/^/m', str_repeat (' ', $indent), $msg);
			echo $msg;
			return;
		}

		$no = count ($msg);
		$i = 1;
		foreach ( $msg as $v ) {
			if ( $i++ !== $no )
				$v = rtrim ($v) . "\n";
			printf ("%s%s", str_repeat (' ', $indent), $v);
		}
	}
	// }}}

	// {{{ function wordwrap ($msg, $width = 75, $break = "\n", $cut = 0)
	/**
	 * Wraps a string to a given number of characters. Difference with wordwarp of
	 * PHP, wrapped line joins next line and rewraps.
	 * @access	public
	 * @return	strings
	 * @param	string	input string
	 * @param	integer	(optional) The column widht. Defaults to 75
	 * @param	string	(optional) The line is broken using the optional break parameter.
	 *					Defaults to '\n'.
	 * @param	integer	(optional) If the cut is set to TRUE, the string is always wrapped
	 *					at or before the specified width. So if you have a word
	 *					that is larger than the given width, it is broken apart.
	 *					Seealso wordwrap of php
	 */
	function wordwrap ($msg, $width = 75, $break = "\n", $cut = 0) {
		$msg = wordwrap ($msg, $width, $break, $cut);

		$_msg = split ("[{$break}]", $msg);
		$_msgl = count ($_msg);

		for ( $i=0; $i<$_msgl; $i++ ) {
			$current = rtrim ($_msg[$i]);
			$l = strlen ($current);
			$chk = $width - $l;

			if ( $i == 0 ) {
				$v = $current . "\n";
				continue;
			}

			if ( $chk <= ($width * 0.2) || $chk == $width ) {
				$v .= $current . "\n";
			} else {
				if ( $i == ($_msgl - 1 ) ) {
					$v .= $current . "\n"; 
					break;
				}

				$next = rtrim ($_msg[++$i]);

				if ( ! strlen ($next) ) {
					$v .= "\n";
					continue;
				}

				$current .= ' ' . $next;

				$current = wordwrap ($current, $width, $break, $cut);
				$_c = split ("[{$break}]", $current);
				$_cl = count ($_c);

				$v .= $_c[0] . "\n";

				if ( $_cl > 1 )
					$_msg[--$i] = $_c[1];
			}
		}

		return $v;
	}
	// }}}

	// {{{ function warning ($msg, $type = self::EPRINT_WARNING)
	/**
	 * Make php user warning/error message of eString or any Pear.
	 * @access	public
	 * @return	void
	 * @param	string	Input string. If prefix of message is started with
	 * 					'^[^:]+::', prefix replace pear name of error messages.
	 * @param	integer	(optional) Defaults to EPRINT_WARNING<br>
	 * 					EPRINT_WARNING : make php warning<br>
	 * 					EPRINT_ERROR   : make php error
	 */
	function warning ($msg, $type = self::EPRINT_WARNING) {
		$t = ($type === self::EPRINT_WARNING) ? 'Warnging' : 'Error';

		$dinfo = debug_backtrace ();
		$file = $dinfo[1]['file'];
		$line = $dinfo[1]['line'];
		$postfix = sprintf (' in %s on line %d', $file, $line);

		if ( preg_match ('/^([^:\s]+)::(.+)/', $msg, $matches) ) {
			$pearName = $matches[1];
			$msg = $matches[2];
		} else
			$pearName = 'ePrint';

		$prefix = "PEAR::${pearName} {$t}: ";
		$format = $prefix . $msg . $postfix;

		self::ePrintf ($format);

		if ( $type === self::EPRINT_ERROR )
			exit (1);
	}
	// }}}

	/*
	 * Ansi print
	 */

	// {{{ function setColor ($color)
	/**
	 * return Ansi code about color string
	 * @access	public
	 * @return	string	defaults to gray code (^[[1;30m)
	 * @param	string	(optional) Color strings for ansi code. Defaults to GRAY
	 * 					You can use follows.<br>
	 * 					BLUE,<br>
	 * 					CYAN,<br>
	 * 					GRAY,<br>
	 * 					GREEN,<br>
	 * 					MEGENTA,<br>
	 * 					RED,<br>
	 * 					WHITE,<br>
	 * 					YELLOW,<br>
	 * 					END.<br>
	 * 					This value is not case sencitive.
	 */
	function setColor ($color) {
		$color = $color ? strtolower ($color): 'gray';

		switch ( $color ) {
			case 'blue'    : return '[1;34m'; break;
			case 'cyan'    : return '[1;36m'; break;
			case 'gray'    : return '[1;30m'; break;
			case 'green'   : return '[1;32m'; break;
			case 'megenta' : return '[1;35m'; break;
			case 'red'     : return '[1;31m'; break;
			case 'white'   : return '[1;37m'; break;
			case 'yellow'  : return '[1;33m'; break;
			case 'end'     : return '[7;0m';  break;
		}

		/* defaults to gray code */
		return '[1;30m';
	}
	// }}}

	// {{{ function aPrintf ()
	/**
	 * print ansi formatted string
	 * @access	public
	 * @return	int		length of print string
	 * @param	string	$color	color string for anci code
	 * @param	string	$format	same format of printf
	 * @param	string	$args	(optional) format value.
	 * @param	string	$...	(optional) format value.
	 */
	function aPrintf () {
		$args= func_get_args ();

		if ( count ($args) < 2 ) {
			self::warning ('Wrong parameter count for ePrint::aPrintf()');
			return;
		}

		$color = array_shift ($args);
		$start = self::setColor ($color);
		$end   = self::setColor	('end');

		$format = array_shift ($args);
		$format = '%s' . $format . '%s';
		$_args = array_merge (array ($start), $args, array ($end));

		return vprintf ($format, $_args);
	}
	// }}}

	// {{{ function asPrintf ()
	/**
	 * return ansi formatted string
	 * @access	public
	 * @return	string
	 * @param	string	$color	color string for anci code
	 * @param	string	$format	same format of printf
	 * @param	string	$args	(optional) format value.
	 * @param	string	$...	(optional) format value.
	 */
	function asPrintf () {
		$args= func_get_args ();

		if ( count ($args) < 2 ) {
			self::warning ('Wrong parameter count for ePrint::aPrintf()');
			return;
		}

		$color = array_shift ($args);
		$start = self::setColor ($color);
		$end   = self::setColor	('end');

		$format = array_shift ($args);
		$format = '%s' . $format . '%s';
		$_args = array_merge (array ($start), $args, array ($end));

		return vsprintf ($format, $_args);
	}
	// }}}

	// {{{ function unregist ()
	/**
	 * Unregist stdin/stdout/stderr file descriptor
	 * @access	public
	 * @return	void
	 */
	function unregist () {
		if ( is_resource (self::$stdin) )
			fclose (self::$stdin);
		if ( is_resource (self::$stdout) )
			fclose (self::$stdout);
		if ( is_resource (self::$stderr) )
			fclose (self::$stderr);
	}
	// }}}

	// {{{ destructor
	function __destruct () {
		self::unregist();
	}
	// }}}
}

?>
